package com.ruoyi.project.pssr.mapper;

import java.util.List;
import com.ruoyi.framework.aspectj.lang.annotation.DataScope;
import com.ruoyi.project.pssr.domain.TPssrSafetyValve;

/**
 * 安全设施-安全阀Mapper接口
 *
 * @author ssy
 * @date 2024-09-18
 */
public interface TPssrSafetyValveMapper
{
    /**
     * 查询安全设施-安全阀
     *
     * @param id 安全设施-安全阀ID
     * @return 安全设施-安全阀
     */
    public TPssrSafetyValve selectTPssrSafetyValveById(Long id);
    public int selectNoConfirmCount(Long subId);

    /**
     * 查询安全设施-安全阀列表
     *
     * @param tPssrSafetyValve 安全设施-安全阀
     * @return 安全设施-安全阀集合
     */
    
    public List<TPssrSafetyValve> selectTPssrSafetyValveList(TPssrSafetyValve tPssrSafetyValve);

    /**
     * 新增安全设施-安全阀
     *
     * @param tPssrSafetyValve 安全设施-安全阀
     * @return 结果
     */
    public int insertTPssrSafetyValve(TPssrSafetyValve tPssrSafetyValve);

    /**
     * 修改安全设施-安全阀
     *
     * @param tPssrSafetyValve 安全设施-安全阀
     * @return 结果
     */
    public int updateTPssrSafetyValve(TPssrSafetyValve tPssrSafetyValve);
    public int updateTPssrSafetyValveByIds(TPssrSafetyValve tPssrSafetyValve);

    public int updateAllBySubId(TPssrSafetyValve tPssrSafetyValve);

    /**
     * 删除安全设施-安全阀
     *
     * @param id 安全设施-安全阀ID
     * @return 结果
     */
    public int deleteTPssrSafetyValveById(Long id);

    /**
     * 批量删除安全设施-安全阀
     *
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    public int deleteTPssrSafetyValveByIds(Long[] ids);
}
